@@ -1,28 +1,37 @@
 from functools import wraps
 from xml.etree.ElementTree import XML
 import json
-import requests
 
-__version__ = "0.5"
+__version__ = "0.6"
 
 ALL = '*'
 ENDPOINT_URL = 'http://api.census.gov/data/%s/%s'
 DEFINITIONS = {
     'acs5': {
-        '2011': 'http://www.census.gov/developers/data/acs_5yr_2011_var.xml',
-        '2010': 'http://www.census.gov/developers/data/acs_5yr_2010_var.xml',
+        '2011': 'http://api.census.gov/data/2011/acs5/variables.xml',
+        '2010': 'http://api.census.gov/data/2010/acs5/variables.xml',
+    },
+    'acs1/profile': {
+        '2012': 'http://api.census.gov/data/2012/acs1/variables.xml',
     },
     'sf1': {
-        '2010': 'http://www.census.gov/developers/data/sf1.xml',
-        '2000': 'http://www.census.gov/developers/data/2000_sf1.xml',
-        '1990': 'http://www.census.gov/developers/data/1990_sf1.xml',
+        '2010': 'http://api.census.gov/data/2010/sf1/variables.xml',
+        '2000': 'http://api.census.gov/data/2000/sf1/variables.xml',
+        '1990': 'http://api.census.gov/data/1990/sf1/variables.xml',
     },
     'sf3': {
-        '2000': 'http://www.census.gov/developers/data/2000_sf3.xml',
-        '1990': 'http://www.census.gov/developers/data/1990_sf3.xml',
+        '2000': 'http://api.census.gov/data/2000/sf3/variables.xml',
+        '1990': 'http://api.census.gov/data/1990/sf3/variables.xml',
     },
 }
 
+class APIKeyError(Exception):
+    '''Invalid API Key'''
+    def __init__(self, value):
+        self.value = value
+    def __str__(self):
+        return repr(self.value)
+
 
 def list_or_str(v):
     """ Convert a single value into a list.
@@ -54,9 +63,10 @@ class UnsupportedYearException(CensusException):
 
 class Client(object):
 
-    def __init__(self, key, year=None):
-        self._session = requests.session()
+    def __init__(self, key, year=None, session=None):
+        import requests
         self._key = key
+        self.session = session or requests.session()
         if year:
             self.default_year = year
 
@@ -119,11 +129,16 @@ class Client(object):
         headers = {
             'User-Agent': 'python-census/%s github.com/sunlightlabs/census' % __version__
         }
-        resp = self._session.get(url, params=params, headers=headers)
+        resp = self.session.get(url, params=params, headers=headers)
 
         if resp.status_code == 200:
-
-            data = json.loads(resp.text)
+            try:
+                data = json.loads(resp.text)
+            except ValueError as ex:
+                if '<title>Invalid Key</title>' in resp.text:
+                    raise APIKeyError(' '.join(resp.text.splitlines()))
+                else:
+                    raise ex
 
             headers = data[0]
             return [dict(zip(headers, d)) for d in data[1:]]
@@ -135,7 +150,7 @@ class Client(object):
             raise CensusException(resp.text)
 
 
-class ACSClient(Client):
+class ACS5Client(Client):
 
     default_year = 2011
     dataset = 'acs5'
@@ -191,6 +206,27 @@ class ACSClient(Client):
             'for': 'zip code tabulation area:%s' % zcta,
         }, **kwargs)
 
+class ACS1DpClient(Client):
+
+    default_year = 2012
+    dataset = 'acs1/profile'
+
+    @supported_years(2012)
+    def us(self, fields, **kwargs):
+        return self.get(fields, geo={'for': 'us:1'}, **kwargs)
+
+    @supported_years(2012)
+    def state(self, fields, state_fips, **kwargs):
+        return self.get(fields, geo={
+            'for': 'state:%s' % state_fips,
+        }, **kwargs)
+
+    @supported_years(2012)
+    def state_district(self, fields, state_fips, district, **kwargs):
+        return self.get(fields, geo={
+            'for': 'congressional district:%s' % district,
+            'in': 'state:%s' % state_fips,
+        }, **kwargs)
 
 class SF1Client(Client):
 
@@ -304,7 +340,14 @@ class Census(object):
 
     ALL = ALL
 
-    def __init__(self, key, year=None):
-        self.acs = ACSClient(key, year)
-        self.sf1 = SF1Client(key, year)
-        self.sf3 = SF3Client(key, year)
+    def __init__(self, key, year=None, session=None):
+        import requests
+
+        if not session:
+            session = requests.session()
+
+        self.acs = ACS5Client(key, year, session)
+        self.acs5 = ACS5Client(key, year, session)
+        self.acs1dp = ACS1DpClient(key, year, session)
+        self.sf1 = SF1Client(key, year, session)
+        self.sf3 = SF3Client(key, year, session)
